@@ -2,33 +2,12 @@
 Tests for the Pyzotero module
 
 This file is part of Pyzotero.
-
-The MIT License (MIT)
-
-Copyright (c) 2015 Stephan Hügel
-
-Permission is hereby granted, free of charge, to any person obtaining a copy
-of this software and associated documentation files (the "Software"), to deal
-in the Software without restriction, including without limitation the rights
-to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-copies of the Software, and to permit persons to whom the Software is
-furnished to do so, subject to the following conditions:
-
-The above copyright notice and this permission notice shall be included in
-all copies or substantial portions of the Software.
-
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-THE SOFTWARE.
 """
+# ruff: noqa: N802
 
 import os
-import unittest
 import time
+import unittest
 
 import httpretty
 from dateutil import parser
@@ -39,7 +18,7 @@ try:
 except ModuleNotFoundError:
     from pyzotero import zotero as z
 
-from urllib.parse import parse_qs
+from urllib.parse import parse_qs, urlencode
 
 
 class ZoteroTests(unittest.TestCase):
@@ -48,8 +27,8 @@ class ZoteroTests(unittest.TestCase):
     cwd = os.path.dirname(os.path.realpath(__file__))
 
     def get_doc(self, doc_name, cwd=cwd):
-        """return the requested test document"""
-        with open(os.path.join(cwd, "api_responses", "%s" % doc_name), "r") as f:
+        """Return the requested test document"""
+        with open(os.path.join(cwd, "api_responses", f"{doc_name}")) as f:
             return f.read()
 
     def setUp(self):
@@ -82,12 +61,19 @@ class ZoteroTests(unittest.TestCase):
             body=self.items_doc,
         )
 
+    def testBuildUrlCorrectHandleEndpoint(self):
+        """Url should be concat correctly by build_url"""
+        url = z.build_url("http://localhost:23119/api", "/users/0")
+        self.assertEqual(url, "http://localhost:23119/api/users/0")
+        url = z.build_url("http://localhost:23119/api/", "/users/0")
+        self.assertEqual(url, "http://localhost:23119/api/users/0")
+
     @httpretty.activate
     def testFailWithoutCredentials(self):
         """Instance creation should fail, because we're leaving out a
         credential
         """
-        with self.assertRaises(z.ze.MissingCredentials):
+        with self.assertRaises(z.ze.MissingCredentialsError):
             z.Zotero("myuserID")
 
     @httpretty.activate
@@ -96,22 +82,42 @@ class ZoteroTests(unittest.TestCase):
         zot = z.Zotero("myuserID", "user", "myuserkey")
         zot.add_parameters(limit=0, start=7)
         self.assertEqual(
-            parse_qs("start=7&limit=100&format=json"), parse_qs(zot.url_params)
+            parse_qs("start=7&limit=100&format=json"),
+            parse_qs(urlencode(zot.url_params, doseq=True)),
         )
 
+    @httpretty.activate
+    def testLocale(self):
+        """Should correctly add locale to request because it's an initial request"""
+        HTTPretty.register_uri(
+            HTTPretty.GET,
+            "https://api.zotero.org/users/myuserID/items",
+            content_type="application/json",
+            body=self.item_doc,
+        )
+        zot = z.Zotero("myuserID", "user", "myuserkey")
+        _ = zot.items()
+        req = zot.request
+        self.assertEqual(str(req.url).find("locale"), 44)
+
     @httpretty.activate
     def testRequestBuilderLimitNone(self):
         """Should skip limit = 100 param if limit is set to None"""
         zot = z.Zotero("myuserID", "user", "myuserkey")
         zot.add_parameters(limit=None, start=7)
-        self.assertEqual(parse_qs("start=7&format=json"), parse_qs(zot.url_params))
+        self.assertEqual(
+            parse_qs("start=7&format=json"), parse_qs(urlencode(zot.url_params))
+        )
 
     @httpretty.activate
     def testRequestBuilderLimitNegativeOne(self):
         """Should skip limit = 100 param if limit is set to -1"""
         zot = z.Zotero("myuserID", "user", "myuserkey")
         zot.add_parameters(limit=-1, start=7)
-        self.assertEqual(parse_qs("start=7&format=json"), parse_qs(zot.url_params))
+        self.assertEqual(
+            parse_qs("start=7&format=json"),
+            parse_qs(urlencode(zot.url_params, doseq=True)),
+        )
 
     # @httpretty.activate
     # def testBuildQuery(self):
@@ -141,12 +147,12 @@ class ZoteroTests(unittest.TestCase):
             body=self.item_doc,
         )
         items_data = zot.items()
-        self.assertEqual(u"X42A7DEE", items_data["data"]["key"])
+        self.assertEqual("X42A7DEE", items_data["data"]["key"])
         self.assertEqual(
-            u"Institute of Physics (Great Britain)",
+            "Institute of Physics (Great Britain)",
             items_data["data"]["creators"][0]["name"],
         )
-        self.assertEqual(u"book", items_data["data"]["itemType"])
+        self.assertEqual("book", items_data["data"]["itemType"])
         test_dt = parser.parse("2011-01-13T03:37:29Z")
         incoming_dt = parser.parse(items_data["data"]["dateModified"])
         self.assertEqual(test_dt, incoming_dt)
@@ -194,13 +200,13 @@ class ZoteroTests(unittest.TestCase):
             body=self.attachments_doc,
         )
         attachments_data = zot.items()
-        self.assertEqual(u"1641 Depositions", attachments_data["data"]["title"])
+        self.assertEqual("1641 Depositions", attachments_data["data"]["title"])
 
     @httpretty.activate
     def testParseKeysResponse(self):
         """Check that parsing plain keys returned by format = keys works"""
         zot = z.Zotero("myuserid", "user", "myuserkey")
-        zot.url_params = "format=keys"
+        zot.url_params = {"format": "keys"}
         HTTPretty.register_uri(
             HTTPretty.GET,
             "https://api.zotero.org/users/myuserid/items?format=keys",
@@ -251,7 +257,7 @@ class ZoteroTests(unittest.TestCase):
             body=self.items_doc,
         )
         items_data = zot.children("ABC123")
-        self.assertEqual(u"NM66T6EF", items_data[0]["key"])
+        self.assertEqual("NM66T6EF", items_data[0]["key"])
 
     @httpretty.activate
     def testCitUTF8(self):
@@ -265,7 +271,7 @@ class ZoteroTests(unittest.TestCase):
             body=self.citation_doc,
         )
         cit = zot.item("GW8V2CK7", content="citation", style="chicago-author-date")
-        self.assertEqual(cit[0], u"<span>(Ans\\xe6lm and Tka\\u010dik 2014)</span>")
+        self.assertEqual(cit[0], "<span>(Ans\\xe6lm and Tka\\u010dik 2014)</span>")
 
     # @httpretty.activate
     # def testParseItemAtomBibDoc(self):
@@ -343,7 +349,23 @@ class ZoteroTests(unittest.TestCase):
             body=self.tags_doc,
         )
         tags_data = zot.tags()
-        self.assertEqual(u"Community / Economic Development", tags_data[0])
+        self.assertEqual("Community / Economic Development", tags_data[0])
+
+    @httpretty.activate
+    def testUrlBuild(self):
+        """Ensure that URL parameters are successfully encoded by the http library"""
+        zot = z.Zotero("myuserID", "user", "myuserkey")
+        HTTPretty.register_uri(
+            HTTPretty.GET,
+            "https://api.zotero.org/users/myuserID/tags?limit=1",
+            content_type="application/json",
+            body=self.tags_doc,
+        )
+        _ = zot.tags(limit=1)
+        self.assertEqual(
+            "https://api.zotero.org/users/myuserID/tags?locale=en-US&limit=1&format=json",
+            zot.request.url,
+        )
 
     @httpretty.activate
     def testParseLinkHeaders(self):
@@ -388,7 +410,8 @@ class ZoteroTests(unittest.TestCase):
         zot._build_query("/whatever")
         zot.add_parameters(start=2)
         self.assertEqual(
-            parse_qs("start=2&format=json&limit=100"), parse_qs(zot.url_params)
+            parse_qs("start=2&format=json&limit=100"),
+            parse_qs(urlencode(zot.url_params, doseq=True)),
         )
 
     @httpretty.activate
@@ -415,7 +438,21 @@ class ZoteroTests(unittest.TestCase):
             body=self.items_doc,
             status=403,
         )
-        with self.assertRaises(z.ze.UserNotAuthorised):
+        with self.assertRaises(z.ze.UserNotAuthorisedError):
+            zot.items()
+
+    @httpretty.activate
+    def testTimeout(self):
+        """Ensure that an error is properly raised for 503"""
+        zot = z.Zotero("myuserID", "user", "myuserkey")
+        HTTPretty.register_uri(
+            HTTPretty.GET,
+            "https://api.zotero.org/users/myuserID/items",
+            content_type="application/json",
+            body=self.items_doc,
+            status=503,
+        )
+        with self.assertRaises(z.ze.HTTPError):
             zot.items()
 
     @httpretty.activate
@@ -429,7 +466,7 @@ class ZoteroTests(unittest.TestCase):
             body=self.items_doc,
             status=400,
         )
-        with self.assertRaises(z.ze.UnsupportedParams):
+        with self.assertRaises(z.ze.UnsupportedParamsError):
             zot.items()
 
     @httpretty.activate
@@ -443,7 +480,7 @@ class ZoteroTests(unittest.TestCase):
             content_type="application/json",
             status=404,
         )
-        with self.assertRaises(z.ze.ResourceNotFound):
+        with self.assertRaises(z.ze.ResourceNotFoundError):
             zot.items()
 
     @httpretty.activate
@@ -490,7 +527,7 @@ class ZoteroTests(unittest.TestCase):
         """Ensure that collection creation fails with the wrong dict"""
         zot = z.Zotero("myuserID", "user", "myuserkey")
         t = [{"foo": "bar"}]
-        with self.assertRaises(z.ze.ParamNotPassed):
+        with self.assertRaises(z.ze.ParamNotPassedError):
             t = zot.create_collections(t)
 
     @httpretty.activate
@@ -689,7 +726,7 @@ class ZoteroTests(unittest.TestCase):
         """Should fail because we're passing too many items"""
         itms = [i for i in range(51)]
         zot = z.Zotero("myuserID", "user", "myuserkey")
-        with self.assertRaises(z.ze.TooManyItems):
+        with self.assertRaises(z.ze.TooManyItemsError):
             zot.create_items(itms)
 
     @httpretty.activate
